﻿//--------------------------------------------------------------
//
// Eamon Revolutions Role Playing Game
// Derek C. Jeter
//
//--------------------------------------------------------------

function XCmdHandlerAttack()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var result = ctx.gameEngine.targetSearch(cmd);

        if (result == null)
        {
            return CmdResultFactory.error();
        }

        if (ctx.gameStateData.runChar.weaponNumber <= 0)
        {
            ctx.console.writeLine("You must first READY a weapon!");

            return CmdResultFactory.error();
        }

        var monster = result.target;

        if (monster.personality != CPersonalityType.enemy)
        {
            //TODO: Check if we're attacking a friend    
        }

        //TODO: Implement combat routine
        ctx.console.writeLine("This command has not been implemented!", "red");
        
        ctx.combatEngine.doAttack(ctx.gameStateData.runChar, monster);

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerClose()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var closeableArtifactTypes = [CArtifactType.container, CArtifactType.drinkable, CArtifactType.edible, CArtifactType.readable, CArtifactType.doorGate];

        var artifact = ctx.gameEngine.typedArtifactSearch(cmd, closeableArtifactTypes, [CLocationType.carriedByPlayer, CLocationType.room, CLocationType.embedded]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (artifact.type == CArtifactType.drinkable || artifact.type == CArtifactType.edible || artifact.type == CArtifactType.readable || artifact.openStatus == COpenStatus.disallow)
        {
            ctx.console.writeLine("You don't need to.");
            return CmdResultFactory.error();
        }

        if (artifact.type == CArtifactType.doorGate && artifact.isHidden)
        {
            ctx.console.writeLine("I don't follow you.");
            return CmdResultFactory.error();
        }

        if (artifact.openStatus == COpenStatus.smashed)
        {
            ctx.console.writeLine("You broke it!");
            return CmdResultFactory.error();
        }

        if (artifact.openStatus == COpenStatus.closed)
        {
            ctx.console.writeLine("It's not open.");
            return CmdResultFactory.error();
        }

        artifact.openStatus = COpenStatus.closed;

        ctx.console.writeLine(artifact.name.capitalize() +  " closed.");

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerDrinkEat(m)
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;
    this.artifactType = m;

    this.execute = function (ctx, cmd)
    {
        var artifact = ctx.gameEngine.typedArtifactSearch(cmd, [this.artifactType], [CLocationType.carriedByPlayer, CLocationType.room, CLocationType.embedded]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (artifact.openStatus == COpenStatus.closed)
        {
            ctx.console.writeLine("You must first open it.");
            return CmdResultFactory.error();
        }

        if (artifact.amountRemaining < 1)
        {
            ctx.console.writeLine("There's none left.");
            return CmdResultFactory.error();
        }

        artifact.amountRemaining--;

        if (artifact.healPoints == 0)
        {
            ctx.console.writeLine("Ok.");
        }
        else
        {
            var feel = (artifact.healPoints > 0) ? "better" : "worse";

            ctx.console.writeLine("You feel " + feel + "!");

            ctx.gameEngine.adjustDamage(ctx.gameStateData.runChar, artifact.healPoints);
        }

        if (artifact.amountRemaining < 1 && this.artifactType == CArtifactType.edible)
        {
            artifact.locationDetail.type = CLocationType.hidden;
            artifact.locationDetail.number = 0;

            ctx.console.writeLine("You eat it all.");
        }

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerDrop()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var artifact = ctx.gameEngine.inventoryArtifactSearch(cmd);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (artifact.locationDetail.CLocationType == CLocationType.wornByPlayer)
        {
            ctx.console.writeLine("You're wearing it. REMOVE it first.");
            return CmdResultFactory.error();
        }

        // If this was the player's readied weapon, then un-ready it
        if (ctx.gameStateData.runChar.weaponNumber == artifact.number)
        {
            ctx.gameStateData.runChar.weaponNumber = 0;
        }

        artifact.locationDetail.type = CLocationType.room;
        artifact.locationDetail.number = ctx.gameStateData.room;

        ctx.console.writeLine(artifact.name.capitalize() + " dropped.");

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerExamine()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        // Examine room
        if (cmd.directObject == "room")
        {
            this.examineRoom();
        }
        else
        {
            if (!this.examineArtifacts(cmd, ctx))
            {
                if (!this.examineMonsters(cmd, ctx))
                {
                    ctx.console.writeLine("You see nothing special.");
                }
            }
        }

        return CmdResultFactory.ok();
    }

    this.examineRoom = function(ctx)
    {
        var room = ctx.gameEngine.getRoom(ctx.gameStateData.room);

        ctx.gameEngine.printDescription(room.description, "white");
    }

    this.examineArtifacts = function(cmd, ctx)
    {
        var matches = ctx.gameEngine.artifactSearch(cmd.directObject, [CLocationType.carriedByPlayer, CLocationType.wornByPlayer, CLocationType.room, CLocationType.embedded]);

        if (matches.length == 0)
        {
            return false;
        }

        if (matches.length > 1)
        {
            ctx.console.writeLine("Please be more specific!");
            return true;
        }

        var artifact = matches[0];

        // If the artifact was embedded, place it in the room
        if (artifact.locationDetail.type == CLocationType.embedded)
        {
            artifact.locationDetail.type = CLocationType.room;
            artifact.locationDetail.number = ctx.gameStateData.room;
        }

        // If the artifact was a hidden passage, it is now visible
        if (artifact.type == CArtifactType.doorGate)
        {
            artifact.isHidden = false;
        }

        ctx.gameEngine.printDescription(artifact.description, "white");
        artifact.hasBeenSeen = true;

        if (artifact.type == CArtifactType.edible || artifact.type == CArtifactType.drinkable)
        {
            var x = (artifact.type == CArtifactType.edible) ? "bite" : "swallow";
            var s = (artifact.amountRemaining > 1) ? "s" : "";
            var v = (artifact.amountRemaining > 1) ? "are" : "is";

            ctx.console.writeBlankLine();
            ctx.console.writeLine("There " + v + " " + artifact.amountRemaining + " " + x + s + " left.");
        }

        return true;
    }

    this.examineMonsters = function(cmd, ctx)
    {
        var results = _.filter(ctx.gameStateData.gameModule.monsters, function(x)
        {
            return (x.locationDetail.type == CLocationType.room && x.locationDetail.number == ctx.gameStateData.room &&
                x.name.toLowerCase().indexOf(cmd.directObject) > -1);
        });

        if (results.length == 0)
        {
            return false;
        }

        if (results.length > 1)
        {
            ctx.console.writeLine("Please be more specific!");
            return true;
        }

        var monster = results[0];

        ctx.gameEngine.printDescription(monster.description, "white");

        ctx.gameEngine.printHealthStatus(monster, false);
        
        return true;
    }
};

function XCmdHandlerFree()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var artifact = ctx.gameEngine.typedArtifactSearch(cmd, [CArtifactType.boundMonster], [CLocationType.room, CLocationType.embedded]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (cmd.indirectObject == '')
        {
            ctx.console.writeLine("Free " + artifact.name + " with what?");

            return CmdResultFactory.error();
        }

        if (artifact.guardMonsterNumber > 0)
        {
            var guard = ctx.gameEngine.getMonster(artifact.guardMonsterNumber);

            if (guard != null && guard.locationDetail.type == CLocationType.room && guard.locationDetail.number == ctx.gameStateData.room)
            {
                ctx.console.writeLine(ctx.gameEngine.getMonsterName(guard) + " won't let you!");

                return CmdResultFactory.error();
            }
        }

        if (!ctx.gameEngine.checkKey(artifact, cmd))
        {
            return CmdResultFactory.error();
        }

        var monster = ctx.gameEngine.getMonster(artifact.boundMonsterNumber);

        monster.locationDetail.type = CLocationType.room;
        monster.locationDetail.number = ctx.gameStateData.room;

        ctx.console.writeLine("You have freed " + ctx.gameEngine.getMonsterName(monster) + ".");

        return CmdResultFactory.ok();
    };
}

function XCmdHandlerGet()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function (ctx, cmd)
    {
        var matches = ctx.gameEngine.artifactSearch(cmd.directObject, [ CLocationType.room, CLocationType.embedded ]);

        if (matches.length == 0)
        {
            ctx.console.writeLine("You can't get that.");
            return CmdResultFactory.error();
        }

        if (matches.length > 1)
        {
            ctx.console.writeLine("Please be more specific!");
            return CmdResultFactory.error();
        }

        var artifact = matches[0];

        if (!ctx.gameEngine.checkItemGet(artifact))
        {
            return CmdResultFactory.error();
        }

        artifact.locationDetail.type = CLocationType.carriedByPlayer;
        artifact.locationDetail.number = 0;

        ctx.console.writeLine(artifact.name.capitalize() + " taken.");

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerGive()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (cmd.indirectObject == '')
        {
            ctx.console.writeLine("To whom?");

            return CmdResultFactory.error();
        }

        var monster = ctx.gameEngine.monsterSearch(cmd.indirectObject);

        if (monster == null)
        {
            return CmdResultFactory.error();
        }

        var gold = parseInt(cmd.directObject);

        if (isNaN(gold))
        {
            var whereToLook = [CLocationType.room, CLocationType.embedded, CLocationType.carriedByPlayer, CLocationType.wornByPlayer];
            var matches = ctx.gameEngine.artifactSearch(cmd.directObject, whereToLook);

            if (matches.length == 0)
            {
                ctx.console.writeLine("You can't give that.");
                return CmdResultFactory.error();
            }

            if (matches.length > 1)
            {
                ctx.console.writeLine("Please be more specific!");
                return CmdResultFactory.error();
            }

            return this.giveItem(ctx, monster, matches[0]);
        }
        else
        {
            return this.giveGold(ctx, monster, gold);
        }
    };

    this.giveGold = function(ctx, monster, amount)
    {
        if (ctx.gameStateData.runChar.gold < amount)
        {
            ctx.console.writeLine("You only have" + ctx.gameStateData.runChar.gold + " gold pieces.");

            return CmdResultFactory.ok();
        }

        ctx.gameStateData.runChar.gold -= amount;

        ctx.console.writeLine(ctx.gameEngine.getMonsterName(monster) + " takes the gold...");

        if (monster.personality == CPersonalityType.neutral && amount >= 5000)
        {
            monster.personality = CPersonalityType.friend;

            ctx.gameEngine.printMonsterPersonalityResponse(monster);
        }

        return CmdResultFactory.ok();
    };

    this.giveItem = function(ctx, monster, artifact)
    {
        // If monster is an enemy or monster is neutral and we're giving something worth less than 5k, just print personality response
        if (monster.personality == CPersonalityType.enemy || (monster.personality == CPersonalityType.neutral && artifact.value < 5000))
        {
            ctx.gameEngine.printMonsterPersonalityResponse(monster);

            return CmdResultFactory.ok();
        }

        // Dead body - NO THANKS
        if (artifact.type == CArtifactType.deadBody)
        {
            ctx.console.writeLine(ctx.gameEngine.getMonsterName(monster) + " politely refuses.");

            return CmdResultFactory.ok();
        }

        ctx.console.writeLine("You give the " + artifact.name + " to " + ctx.gameEngine.getMonsterName(monster) + ".");

        // Give a weapon
        if ((artifact.type == CArtifactType.weapon || artifact.type == CArtifactType.magicWeapon) && monster.weaponNumber < 0)
        {
            monster.weaponNumber = artifact.number;

            ctx.console.writeLine(ctx.gameEngine.getMonsterName(monster) + " readies " + artifact.name + ".");
        }

        // Give drink or food
        if ((artifact.type == CArtifactType.drinkable || artifact.type == CArtifactType.edible) && artifact.amountRemaining > 0)
        {
            artifact.amountRemaining--;

            var action = (artifact.type == CArtifactType.drinkable) ? "drink" : "bite";

            ctx.console.writeLine(ctx.gameEngine.getMonsterName(monster) + " takes a " + action + " and hands it back.");

            if (artifact.type == CArtifactType.edible && artifact.amountRemaining == 0)
            {
                ctx.console.writeLine(ctx.gameEngine.getMonsterName(monster) + " eats it all.");

                artifact.locationDetail.type = CLocationType.hidden;
                artifact.locationDetail.number = 0;
            }

            if (artifact.healPoints != 0)
            {
                ctx.gameEngine.adjustDamage(monster, artifact.healPoints);
            }
        }

        return CmdResultFactory.ok();
    };
};

function XCmdHandlerHints()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (_.isUndefined(ctx.gameStateData.gameModule.hints) || !_.isArray(ctx.gameStateData.gameModule.hints) || ctx.gameStateData.gameModule.hints.length == 0)
        {
            ctx.console.writeLine("Sorry, no hints are available for this adventure.");

            return CmdResultFactory.error();
        }

        if (cmd.directObject == '')
        {
            this.writeHints(ctx);

            return CmdResultFactory.error();
        }

        var hint = _.findWhere(ctx.gameStateData.gameModule.hints, { number: parseInt(cmd.directObject) });

        if (_.isUndefined(hint))
        {
            this.writeHints(ctx);

            return CmdResultFactory.error();
        }
        
        return CmdResultFactory.error();
    }

    this.writeHints = function(ctx)
    {
        ctx.console.writeLine("Use the HINTS command followed by one of the numbers below to see the answer to the question:");
        ctx.console.writeBlankLine();

        _.each(ctx.gameStateData.gameModule.hints, function(h)
        {
            ctx.console.writeLine(h.number + " - " + h.title);
        });
    }
};

function XCmdHandlerInventory()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (cmd.directObject != "")
        {
            return this.inventoryMonster(ctx, cmd.directObject);
        }

        this.listInventory(ctx, ctx.gameStateData.runChar, CLocationType.carriedByPlayer, CLocationType.wornByPlayer);

        return CmdResultFactory.ok();
    }

    this.inventoryMonster = function(ctx, monsterName)
    {
        var monster = ctx.gameEngine.monsterSearch(monsterName);

        if (monster == null)
        {
            return CmdResultFactory.error();
        }

        this.listInventory(ctx, monster, CLocationType.carriedByMonster, CLocationType.wornByMonster);

        return CmdResultFactory.ok();
    };

    this.listInventory = function(ctx, target, carryLocationType, wearLocationType)
    {
        ctx.console.writeLine(ctx.gameEngine.getMonsterName(target) + " is wearing:");

        var flag = false;
        _.each(ctx.gameStateData.gameModule.artifacts, function(a)
        {
            if (a.locationDetail.type == wearLocationType && a.locationDetail.number == target.number)
            {
                ctx.console.writeLine("&nbsp;&nbsp;- " + a.name);
                flag = true;
            }
        });

        if (!flag)
        {
            ctx.console.writeLine("&nbsp;&nbsp;(Nothing)");
        }

        ctx.console.writeBlankLine();

        ctx.console.writeLine(ctx.gameEngine.getMonsterName(target) + " is carrying:");
        flag = false;

        _.each(ctx.gameStateData.gameModule.artifacts, function(a)
        {
            if (a.locationDetail.type == carryLocationType && a.locationDetail.number == target.number)
            {
                ctx.console.write("&nbsp;&nbsp;- " + a.name);
                flag = true;

                if (a.number == target.weaponNumber)
                {
                    ctx.console.write(" [Ready Weapon]");
                }

                if (a.number == ctx.gameStateData.lightSource)
                {
                    ctx.console.write(" [Providing light]");
                }

                // List container contents if open
                if (a.type == CArtifactType.container && a.openStatus == COpenStatus.open)
                {
                    var contents = _.filter(ctx.gameStateData.gameModule.artifacts, function(x)
                    {
                        return (x.locationDetail.type == CLocationType.containedInArtifact && x.toLocaleString.number == a.number);
                    });

                    if (contents.length > 0)
                    {
                        ctx.console.write("&nbsp;&nbsp;&nbsp;- Contents:");

                        _.each(contents, function(x)
                        {
                            ctx.console.write("&nbsp;&nbsp;&nbsp;- " + x.name);
                        });
                    }
                }

                ctx.console.writeBlankLine();
            }
        });

        if (ctx.gameStateData.runChar.gold > 0)
        {
            ctx.console.writeLine("&nbsp;&nbsp;- " + ctx.gameStateData.runChar.gold + " gold pieces");
            flag = true;
        }

        if (!flag)
        {
            ctx.console.writeLine("&nbsp;&nbsp;(Nothing)");
        }

        ctx.console.writeBlankLine();
    };
};

function XCmdHandlerLight()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var artifact = ctx.gameEngine.typedArtifactSearch(cmd, [CArtifactType.lightable], [CLocationType.carriedByPlayer, CLocationType.room, CLocationType.embedded]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (artifact.lightCounter == 0)
        {
            ctx.console.writeLine("It won't light.");
        }
        else
        {
            if (artifact.number == ctx.gameStateData.lightSource)
            {
                ctx.console.writeLine("It's already lit!");

                return CmdResultFactory.ok();
            }

            ctx.console.writeLine("You've lit the " + artifact.name + ".");

            // If another artifact is acting as light source, extinguish it
            if (ctx.gameStateData.lightSource > 0)
            {
                var currentLightSource = ctx.gameEngine.getArtifact(ctx.gameStateData.lightSource);

                ctx.console.writeLine(currentLightSource.name + " extinguished.");
            }

            ctx.gameStateData.lightSource = artifact.number;
        }

        return CmdResultFactory.okVerbose();
    }
};

function XCmdHandlerExtinguish()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var artifact = ctx.gameEngine.typedArtifactSearch(cmd, [CArtifactType.lightable], [CLocationType.carriedByPlayer, CLocationType.wornByPlayer, CLocationType.room, CLocationType.embedded]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (ctx.gameStateData.lightSource != artifact.number)
        {
            ctx.console.writeLine("It's not lit!");

            return CmdResultFactory.error();
        }

        ctx.gameStateData.lightSource = 0;

        ctx.console.writeLine(artifact.name + " extinguished.");

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerLook()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function (ctx, cmd)
    {
        if (cmd.directObject != "" && cmd.directObject != "room")
        {
            // Delegate to EXAMINE command
            return ctx.gameEngine.commandHandlers["examine"].execute(ctx, cmd);
        }

        var room = ctx.gameEngine.getRoom(ctx.gameStateData.room);

        ctx.gameEngine.printDescription(room.description, "white");
        ctx.console.writeBlankLine();

        return CmdResultFactory.okVerbose();
    }
};

function XCmdHandlerOpen()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var matches = ctx.gameEngine.artifactSearch(cmd.directObject, [CLocationType.room, CLocationType.embedded, CLocationType.carriedByPlayer, CLocationType.wornByPlayer]);

        if (matches.length == 0)
        {
            ctx.console.writeLine("You can't open that.");
            return CmdResultFactory.error();
        }

        if (matches.length > 1)
        {
            ctx.console.writeLine("Please be more specific!");
            return CmdResultFactory.error();
        }

        var artifact = matches[0];

        if (artifact.openStatus == COpenStatus.cannotBeOpened)
        {
            ctx.console.writeLine("You can't open that.");
            return CmdResultFactory.error();
        }

        // If this was an embedded item, reveal it
        if (artifact.locationDetail.type == CLocationType.embedded)
        {
            artifact.locationDetail.type = CLocationType.embedded;
            artifact.locationDetail.room = ctx.gameStateData.room;
        }

        // If this was a disguised monster, reveal it
        if (artifact.type == CArtifactType.disguisedMonster)
        {
            ctx.gameEngine.revealDisguisedMonster();

            return CmdResultFactory.ok();
        }

        if (artifact.openStatus == COpenStatus.open || artifact.openStatus == COpenStatus.smashed)
        {
            ctx.console.writeLine("It's already open.");
            return CmdResultFactory.ok();
        }

        if (artifact.openStatus == COpenStatus.disallow)
        {
            ctx.console.writeLine("It won't open.");
            return CmdResultFactory.ok();
        }

        if (artifact.type == CArtifactType.edible || artifact.type == CArtifactType.drinkable || artifact.type == CArtifactType.readable)
        {
            artifact.openStatus = COpenStatus.open;

            ctx.console.writeLine(artifact.name + " opened.");

            return CmdResultFactory.ok();
        }

        // If this can be smashed open and there is no key, tell player they'll
        // have to open it the hard way
        if (artifact.keyNumber == 0 && artifact.smashOpenValue > 0)
        {
            ctx.console.writeLine("You'll have to force it open.");
            return CmdResultFactory.ok();
        }

        if (cmd.indirectObject == '')
        {
            ctx.console.writeLine("It's locked. You'll need to specify a key with which to open it.");
            return CmdResultFactory.ok();
        }

        if (artifact.keyNumber == 0)
        {
            artifact.openStatus = COpenStatus.open;

            ctx.console.writeLine(artifact.name + " opened.");

            return CmdResultFactory.ok();
        }

        if (!ctx.gameEngine.checkKey(artifact, cmd))
        {
            return CmdResultFactory.ok();
        }

        artifact.openStatus = COpenStatus.open;
        artifact.keyNumber = 0;

        ctx.console.writeLine(artifact.name + " opened with the " + key.name + ".");

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerPut()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (cmd.indirectObject == '')
        {
            ctx.console.writeLine("Put it in what?");

            return CmdResultFactory.error();
        }

        var allowedTypes = [
            CArtifactType.treasure,
            CArtifactType.weapon,
            CArtifactType.magicWeapon,
            CArtifactType.container,
            CArtifactType.lightable,
            CArtifactType.drinkable,
            CArtifactType.readable,
            CArtifactType.edible,
            CArtifactType.wearable
        ];

        var artifact = ctx.gameEngine.inventoryArtifactSearch(cmd, allowedTypes);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        var container = ctx.gameEngine.artifactSearch(cmd.indirectObject, [CLocationType.room, CLocationType.embedded, CLocationType.carriedByPlayer, CLocationType.wornByPlayer]);

        if (container == null)
        {
            ctx.console.writeLine("I don't understand what you are trying to put it into.");

            return CmdResultFactory.error();
        }

        if (container.artifactType != CArtifactType.container || container.number == artifact.number)
        {
            ctx.console.writeLine("I don't follow you.");

            return CmdResultFactory.error();
        }

        if (container.openStatus == COpenStatus.closed)
        {
            ctx.console.writeLine("You must first open it.");

            return CmdResultFactory.error();
        }

        if (container.openStatus == COpenStatus.smashed)
        {
            ctx.console.writeLine("You broke it!");

            return CmdResultFactory.error();
        }

        if (container.maxCanHold < 1)
        {
            ctx.console.writeLine("You don't need to.");

            return CmdResultFactory.error();
        }

        if (artifact.weight == -999 || artifact.weight > container.weight)
        {
            ctx.console.writeLine("It won't fit.");

            return CmdResultFactory.ok();
        }

        // Get items already in container
        var contents = _.filter(ctx.gameStateData.gameModule.artifacts, function(a)
        {
            return (a.locationDetail.type == CLocationType.containedInArtifact && a.toLocaleString.number == container.number);
        });

        // Sum the weights of everything in the container
        var contentsWeight = 0;
        _.each(contents, function(c)
        {
            contentsWeight += c;
        });

        if (contents.length >= container.maxCanHold || artifact.weight + contentsWeight > container.weight)
        {
            ctx.console.writeLine("It's full.");

            return CmdResultFactory.ok();
        }

        // Put artifact into the container
        artifact.locationDetail.type = CLocationType.containedInArtifact;
        artifact.locationDetail.number = container.number;

        ctx.console.writeLine("Done.");

        // If the artifact was the light source, extinguish it
        if (artifact.number == ctx.gameStateData.lightSource)
        {
            ctx.gameStateData.lightSource = 0;

            ctx.console.writeBlankLine();
            ctx.console.writeLine("The " + artifact.name + " goes out.");
        }

        // If the artifact was the player's readied weapon, un-ready it
        if (artifact.number == ctx.gameStateData.runChar.weaponNumber)
        {
            ctx.gameStateData.runChar.weaponNumber = 0;
        }

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerRead()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function (ctx, cmd)
    {
        var artifact = ctx.gameEngine.typedArtifactSearch(cmd, [CArtifactType.readable], [CLocationType.carriedByPlayer, CLocationType.room, CLocationType.embedded]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }
        
        if (artifact.openStatus == COpenStatus.closed)
        {
            ctx.console.writeLine("You must first open it.");
            return CmdResultFactory.error();
        }

        ctx.gameEngine.printEffectSequence(artifact.effectNumber, artifact.numEffects);

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerReady()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var artifact = ctx.gameEngine.inventoryArtifactSearch(cmd, [CArtifactType.wearable, CArtifactType.weapon, CArtifactType.magicWeapon]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (artifact.type == CArtifactType.wear)
        {
            // Delegate to wear command
            return ctx.gameEngine.commandHandlers["wear"].execute(ctx, cmd);
        }

        if (artifact.type != CArtifactType.weapon && artifact.type != CArtifactType.magicWeapon)
        {
            ctx.console.writeLine("That isn't a weapon.");

            return CmdResultFactory.error();
        }

        ctx.gameStateData.runChar.weaponNumber = artifact.number;

        ctx.console.writeLine(artifact.name.capitalize() + " readied.");

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerRemove()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (cmd.indirectObject == '')
        {
            // Remove armor/clothing
            var artifact = ctx.gameEngine.inventoryArtifactSearch(cmd);

            if (artifact == null)
            {
                return CmdResultFactory.error();
            }

            artifact.locationDetail.type = CLocationType.carriedByPlayer;
            artifact.locationDetail.number = 0;

            ctx.console.writeLine(artifact.name.capitalize() + "This command has not been implemented!");

            return CmdResultFactory.ok();
        }

        // Remove from a container
        var matches = ctx.gameEngine.artifactSearch(cmd.indirectObject);

        if (matches != 1)
        {
            ctx.console.writeLine("I don't understand what you are trying to remove it from.");

            return CmdResultFactory.error();
        }

        var container = matches[0];

        if (container.type != CArtifactType.container)
        {
            ctx.console.writeLine("I don't follow you.");

            return CmdResultFactory.error();
        }

        if (container.openStatus != COpenStatus.open)
        {
            ctx.console.writeLine("You must first open it.");

            return CmdResultFactory.error();
        }

        matches = ctx.gameEngine.artifactSearch(cmd.directObject, [CLocationType.containedInArtifact], container.number);

        if (matches != 1)
        {
            ctx.console.writeLine("That isn't in the " + container.name + ".");

            return CmdResultFactory.error();
        }

        var item = matches[0];

        if (!ctx.gameEngine.checkItemGet(item))
        {
            return CmdResultFactory.ok();
        }

        ctx.console.writeLine("Ok.");
        ctx.console.writeBlankLine();

        // If we haven't seen it, write out the full description
        if (!item.hasBeenSeen)
        {
            item.hasBeenSeen = true;

            ctx.gameEngine.printDescription(item.description, "white");
            ctx.console.writeBlankLine();
        }

        return CmdResultFactory.error();
    }
};

function XCmdHandlerRequest()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (cmd.indirectObject == '')
        {
            ctx.console.writeLine("From whom?");

            return CmdResultFactory.error();
        }

        var monster = ctx.gameEngine.monsterSearch(cmd.indirectObject);

        if (monster == null)
        {
            return CmdResultFactory.error();
        }

        var whereToLook = [CLocationType.carriedByMonster, CLocationType.wornByMonster];
        var matches = ctx.gameEngine.artifactSearch(cmd.directObject, whereToLook, monster.number);

        if (matches.length == 0)
        {
            ctx.console.writeLine(ctx.gameEngine.getMonsterName(monster) +  " doesn't have it.");
            return CmdResultFactory.ok();
        }

        if (matches.length > 1)
        {
            ctx.console.writeLine("Please be more specific!");
            return CmdResultFactory.error();
        }

        var artifact = matches[0];

        if (!ctx.gameEngine.checkItemGet(artifact))
        {
            return CmdResultFactory.ok();
        }

        ctx.console.writeLine("Ok.");
        ctx.console.writeBlankLine();

        // If we haven't seen it, write out the full description
        if (!artifact.hasBeenSeen)
        {
            artifact.hasBeenSeen = true;

            ctx.gameEngine.printDescription(artifact.description, "white");
            ctx.console.writeBlankLine();
        }

        // If this is a weapon, ready it
        if (artifact.type == CArtifactType.weapon || artifact.type == CArtifactType.magicWeapon)
        {
            ctx.gameStateData.runChar.weaponNumber = artifact.number;

            ctx.console.writeLine(artifact.name.capitalize() + " readied.");
        }

        return CmdResultFactory.error();
    }
};

function XCmdHandlerSay()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function (ctx, cmd)
    {
        if (cmd.directObject == "")
        {
            ctx.console.writeLine("Say what?");

            return CmdResultFactory.error();
        }

        ctx.console.writeLine("Ok, '" + cmd.directObject.capitalize() + "'.");

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerSmile() // :)
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function (ctx)
    {
        // Get all alive monsters in room
        var monsters = _.filter(ctx.gameStateData.gameModule.monsters, function(x)
        {
            return (x.locationDetail.type == CLocationType.room && x.locationDetail.number == ctx.gameStateData.room);
        });

        if (monsters.length == 0)
        {
            var snappy = (ctx.gameStateData.runChar.gender == CGender.male) ? "toothy grin." : "perfect smile.";
            ctx.console.writeLine("No one is around to notice your " + snappy);

            return CmdResultFactory.ok();
        }

        // Output each monster's reaction
        _.each(monsters, function(m)
        {
            ctx.gameEngine.printMonsterPersonalityResponse(m);
        });
    
        return CmdResultFactory.ok();
    }
};

function XCmdHandlerStatus()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = true;

    this.execute = function(ctx)
    {
        var cnsl = ctx.console;
        var c = ctx.gameStateData.runChar;
        var i = ctx.gameStateData.initChar;
        var a = (c.gender == CGender.male) ? "mighty" : "fair";
        var w = ctx.gameEngine.calculateEncumberance();
        console.log(c);
        cnsl.writeLine("You are the " + a + " " + c.name + ":");
        cnsl.writeBlankLine();

        var html = "<table style='font-size:8pt'>";
        html += "<tr><td>Hardiness:</td><td style='width:50px;'>" + c.hardiness + "</td><td>Damage taken:</td><td>" + c.damageTaken + "</td></tr>";
        html += "<tr><td>Agility:</td><td>" + c.agility + "</td><td>Armor expertise:</td><td>" + c.armorExpertise + "%</td></tr>";
        html += "<tr><td>Weight carried:</td><td>" + w + "</td><td>Max carry:</td><td> " + (c.hardiness * 10) + "</td></tr>";
        html += "<tr><td>Gold in hand:</td><td>" + c.gold + "</td><td>Gold in bank:</td><td>" + c.bank + "</td></tr>";
        html += "</table>";

        cnsl.writeLine(html);

        html = "<table style='font-size:8pt'>";
        html += "<tr><td style='border-bottom:1px white solid'>Weapon abilities:</td><td style='width:50px;border-bottom:1px white solid'>&nbsp;</td><td style='border-bottom:1px white solid;'>Spell abilities:</td><td style='border-bottom:1px white solid'>&nbsp;</td></tr>";
        html += "<tr><td>Axe:</td><td>" + c.weaponAbilities.axe + "%</td><td>Blast:</td><td>" + c.spellAbilities.blast + "% (" + i.spellAbilities.blast + "%)</td></tr>";
        html += "<tr><td>Missile:</td><td>" + c.weaponAbilities.missile + "%</td><td>Heal:</td><td>" + c.spellAbilities.heal + "% (" + i.spellAbilities.heal + "%)</td></tr>";
        html += "<tr><td>Club:</td><td>" + c.weaponAbilities.club + "%</td><td>Speed:</td><td>" + c.spellAbilities.speed + "% (" + i.spellAbilities.speed + "%)</td></tr>";
        html += "<tr><td>Spear:</td><td>" + c.weaponAbilities.spear + "%</td><td>Power:</td><td>" + c.spellAbilities.power + "% (" + i.spellAbilities.power + "%)</td></tr>";
        html += "<tr><td>Sword:</td><td>" + c.weaponAbilities.sword + "%</td><td>&nbsp;</td><td>&nbsp;</td></tr>";
        html += "</table>";

        cnsl.writeLine(html);

        var armor = ctx.gameEngine.getArmorDetails();

        if (armor.armorClass > 0)
        {
            cnsl.writeLine("Armor: " + armor.description);
        }

        if (c.weaponNumber > 0)
        {
            cnsl.writeLine("Readied Weapon: " + ctx.gameEngine.getArtifact(c.weaponNumber).name);
        }

        if (ctx.gameStateData.speedCounter > 0)
        {
            cnsl.writeBlankLine();
            cnsl.writeLine("Your speed spell has " + ctx.gameStateData.speedCounter + " rounds remaining.");
        }

        cnsl.writeBlankLine();
        ctx.gameEngine.printHealthStatus(c, false);

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerUse()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function (ctx, cmd) 
    {
        var useableTypes = [CArtifactType.weapon,
                            CArtifactType.magicWeapon,
                            CArtifactType.drinkable,
                            CArtifactType.edible,
                            CArtifactType.wearable,
                            CArtifactType.disguisedMonster];

        var artifact = ctx.gameEngine.inventoryArtifactSearch(cmd, useableTypes);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        switch(artifact.artifactType)
        {
            case CArtifactType.weapon:
            case CArtifactType.magicWeapon:
                return ctx.gameEngine.commandHandlers["ready"].execute(ctx, cmd);

            case CArtifactType.drinkable:
                return ctx.gameEngine.commandHandlers["drink"].execute(ctx, cmd);

            case CArtifactType.edible:
                return ctx.gameEngine.commandHandlers["eat"].execute(ctx, cmd);

            case CArtifactType.wearable:
                return ctx.gameEngine.commandHandlers["wear"].execute(ctx, cmd);

            case CArtifactType.disguisedMonster:
                ctx.gameEngine.revealDisguisedMonster(artifact);
                break;

            default:
                ctx.console.writeLine("Try a different command.");
                break;
        }

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerWear()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var artifact = ctx.gameEngine.inventoryArtifactSearch(cmd, [CArtifactType.wearable]);

        if (artifact == null)
        {
            return CmdResultFactory.error();
        }

        if (artifact.locationDetail.type == CLocationType.wornByPlayer)
        {
            ctx.console.writeLine("You're already wearing it.");

            return CmdResultFactory.error();
        }

        var armorDetails = ctx.gameEngine.getArmorDetails();
        var msg;

        if (artifact.armorClass > 0)
        {
            if (artifact.armorType == CArmorType.shield)
            {
                if (armorDetails.shieldNumber > 0)
                {
                    msg = "You're already wearing a shield!";
                }
                else
                {
                    msg = artifact.name.capitalize() + " worn.";
                }
            }
            else // Armor
            {
                if (armorDetails.armorNumber > 0)
                {
                    msg = "You're already wearing armor!";
                }
                else
                {
                    msg = artifact.name.capitalize() + " worn.";
                }
            }
        }
        else
        {
            msg = artifact.name.capitalize() + " worn.";
        }

        ctx.console.writeLine(msg);

        return CmdResultFactory.ok();
    }
};

// ----- Spell Commands -----

function XCmdHandlerBlast()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var target = ctx.gameEngine.targetSearch(cmd);

        if (target == null)
        {
            return CmdResultFactory.error();
        }

        /*if (monster.personality != CPersonalityType.enemy)
        {
            //TODO: Check if we're attacking a friend    
        }*/

        if (!ctx.gameEngine.checkSpellCast(cmd.verb))
        {
            return CmdResultFactory.ok();
        }
       
        //TODO: Implement combat routine
        ctx.console.writeLine("This command has not been implemented!", "red");

        ctx.gameEngine.checkSpellAbilityIncrease(cmd.verb);

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerHeal()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        var target;
        if (cmd.directObject == "" || cmd.directObject == "me" || cmd.directObject == ctx.gameStateData.runChar.name)
        {
            target = ctx.gameStateData.runChar;
        }
        else
        {
            target = ctx.gameEngine.monsterSearch(cmd.directObject);

            if (target == null)
            {
                return CmdResultFactory.error();
            }
        }

        if (!ctx.gameEngine.checkSpellCast(cmd.verb))
        {
            return CmdResultFactory.ok();
        }

        var healAmount = ctx.rndNum.roll(1, 12);

        if (target && target.damageTaken > 0)
        {
            ctx.console.writeLine(ctx.gameEngine.getTargetNamePossesive(target) + " health improves!");
        }

        ctx.gameEngine.adjustDamage(target, healAmount);

        ctx.gameEngine.checkSpellAbilityIncrease(cmd.verb);

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerPower()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (!ctx.gameEngine.checkSpellCast(cmd.verb))
        {
            return CmdResultFactory.ok();
        }

        //TODO: Check for custom module programming
        ctx.console.writeLine("You hear a loud sonic boom which echoes all around you!");

        ctx.gameEngine.checkSpellAbilityIncrease(cmd.verb);

        return CmdResultFactory.ok();
    }
};

function XCmdHandlerSpeed()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        if (!ctx.gameEngine.checkSpellCast(cmd.verb))
        {
            return CmdResultFactory.ok();
        }

        if (ctx.gameStateData.speedCounter == 0)
        {
            ctx.gameStateData.runChar.agility *= 2;
        }

        var numRounds = ctx.rndNum.roll(1, 10) + 10;

        ctx.gameStateData.speedCounter += numRounds;

        ctx.console.writeLine("You can feel the new agility flowing through you!");

        ctx.gameEngine.checkSpellAbilityIncrease(cmd.verb);

        return CmdResultFactory.ok();
    }
};

// ----- Navigation Commands -----

function XCmdHandlerFlee()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        // Are there any monsters from which to flee?
        var enemies = _.filter(ctx.gameStateData.gameModule.monsters, function(x)
        {
            return (x.locationDetail.type == CLocationType.room && x.locationDetail.number == ctx.gameStateData.room &&
                x.personality == CPersonalityType.enemy);
        });

        if (enemies.length == 0)
        {
            ctx.console.writeLine("Calm down.");

            return CmdResultFactory.ok();
        }

        // If player has specified a direction, go that way
        var direction;
        var connection;
        if (cmd.directObject != '')
        {
            direction = CDirectionMapper.mapName(cmd.directObject);
            connection = ctx.gameEngine.checkExit(direction);

            if (connection == null)
            {
                return CmdResultFactory.ok();
            }
        }
        else
        {
            var room = ctx.gameEngine.getRoom(ctx.gameStateData.room);
            var exits = [];

            // Build list of exits that are open passages or open doors
            // Cannot flee through a closed or hidden door
            _.each(room.exits, function(x)
            {
                if (x.isDoor == false)
                {
                    exits.push(x);
                }
                else
                {
                    var door = ctx.gameEngine.getArtifact(x.doorArtifactNumber);

                    if (door.isHidden == false && door.openStatus == COpenStatus.open)
                    {
                        exits.push(x);
                    }
                }
            });

            if (exits.length == 0)
            {
                ctx.console.writeLine("There's no place to go!");

                return CmdResultFactory.ok();
            }

            // Pick random direction
            var shuffledExits = ctx.rndNum.shuffle(exits);
            connection = shuffledExits[0].connection;
            direction = shuffledExits[0].direction;
        }

        var msg = (direction == 5 || direction == 6)
                        ? CDirectionMapper.mapDirection(direction) + "ward"
                        : "to the " + CDirectionMapper.mapDirection(direction);

        ctx.console.writeLine("Attempting to flee " + msg + "...", "orange", "", ctx.gameStateData.pauseTime);

        // Roll for successful flee attempt
        var monsterAgility = _.max(enemies, function(m) { return m.agility; }).agility; // Fastest enemy in room
        var pct = 50 + ((ctx.gameStateData.runChar.agility - monsterAgility) * 2);
        ctx.console.log("Flee attempt percentage = " + pct);

        if (ctx.rndNum.checkPct(pct))
        {
            ctx.console.writeLine("Success!");

            ctx.gameEngine.moveMonsters(connection);

            ctx.gameStateData.room = connection;

            return CmdResultFactory.nav();
        }
        else
        {
            ctx.console.writeLine("Attempt thwarted! You must stand and fight.");

            return CmdResultFactory.ok();
        }
    }
};

function XCmdHandlerGo(d)
{
    this.direction = d;
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx)
    {
        // Check for unfriendlies about
        if (_.some(ctx.gameStateData.gameModule.monsters, function(x)
        {
            return (x.personality == CPersonalityType.enemy && x.locationDetail.type == CLocationType.room && x.locationDetail.number == ctx.gameStateData.room);
        }))
        {
            ctx.console.writeLine("You can't do that with unfriendlies about!");

            return CmdResultFactory.ok();
        }

        var connection = ctx.gameEngine.checkExit(this.direction);

        if (connection == null)
        {
            return CmdResultFactory.error();
        }

        // If the light source isn't being carried by player, clear it
        if (ctx.gameStateData.lightSource > 0)
        {
            var lightSource = ctx.gameEngine.getArtifact(ctx.gameStateData.lightSource);

            if (lightSource.locationDetail.type != CLocationType.carriedByPlayer)
            {
                ctx.gameStateData.lightSource = 0;
            }
        }

        ctx.gameEngine.moveMonsters(connection);
 
        ctx.gameStateData.room = connection;

        return CmdResultFactory.nav();
    }
};

// ----- Game Management Commands -----

function XCmdHandlerQuit()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = true;

    this.execute = function(ctx, cmd)
    {
        ctx.console.writeLine("This command has not been implemented!", "red");

        return CmdResultFactory.mgmt();
    }
};

function XCmdHandlerRestore()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = true;

    this.execute = function(ctx, cmd)
    {
        ctx.console.writeLine("This command has not been implemented!", "red");

        return CmdResultFactory.mgmt();
    }
};

function XCmdHandlerSave()
{
    this.requiresDebugMode = false;
    this.canUseWhenDead = false;

    this.execute = function(ctx, cmd)
    {
        ctx.console.writeLine("This command has not been implemented!", "red");

        return CmdResultFactory.mgmt();
    }
};

// ----- Debugging Commands -----

function XCmdHandlerJump()
{
    this.requiresDebugMode = true;
    this.canUseWhenDead = true;

    this.execute = function(ctx, cmd)
    {
        ctx.gameStateData.room = parseInt(cmd.directObject);
        
        return CmdResultFactory.mgmt();
    }
};

function XCmdHandlerKill()
{
    this.requiresDebugMode = true;
    this.canUseWhenDead = true;

    this.execute = function(ctx, cmd)
    {
        var monster = ctx.gameEngine.monsterSearch(cmd.directObject);

        if (monster == null)
        {
            return CmdResultFactory.mgmt();
        }

        monster.locationDetail.type = CLocationType.hidden;
        monster.locationDetail.number = 0;

        ctx.console.writeLine(ctx.gameEngine.getMonsterName(monster) + " killed.", "red");

        return CmdResultFactory.mgmt();
    }
};

function XCmdHandlerAcquire()
{
    this.requiresDebugMode = true;
    this.canUseWhenDead = true;

    this.execute = function(ctx, cmd)
    {
        var items = _.filter(ctx.gameStateData.gameModule.artifacts, function(a)
        {
            return (a.name.toLowerCase().indexOf(cmd.directObject) > -1);
        });

        if (items.length == 0)
        {
            ctx.console.writeLine("That artifact does not exist.", "red");
        }

        if (items.length > 1)
        {
            ctx.console.writeLine("Please be more specific!", "red");
        }

        items[0].locationDetail.type = CLocationType.carriedByPlayer;
        items[0].locationDetail.number = 0;

        ctx.console.writeLine(items[0].name + " acquired.", "red");

        return CmdResultFactory.mgmt();
    }
};
