//--------------------------------------------------------------
//
// Eamon Revolutions Role Playing Game
// Derek C. Jeter
//
//--------------------------------------------------------------

function XController()
{
    var self = this;

    self.gameStateData = {};
    self.persistenceManager = {};
    self.gameEngine = new XGameEngine();
    self.charManager = new XCharacterManager();

    // Setup routes
    self.sammy = $.sammy('#main', function()
    {
        this.notFound = function() { }

        this.get('#/:mid/:cid', function()
        {
            var url = "data/" + this.params['mid'] + ".txt";
            var char = self.charManager.load(this.params['cid']);

            if (char == null)
            {
                console.log("Invalid character ID");
                return;
            }

            self.downloadAndInitModule(url, char);
        });

        this.get('#/:id', function()
        {
            var url = "data/" + this.params['id'] + ".txt";

            self.downloadAndInitModule(url, CCharacterFactory.spawnBrutus());
        });
    });

    self.downloadAndInitModule = function(url, char)
    {
        $.get(url, null, function(data)
        {
            ko.applyBindings(data);

            self.gameStateData.gameModule = data;
            self.gameStateData.initChar = char;
            self.gameStateData.runChar = { number: 0, damageTaken: 0 };
            $.extend(true, self.gameStateData.runChar, self.gameStateData.initChar);
            self.gameStateData.debugMode = true; //TODO: This should be FALSE by default
            self.gameStateData.pauseTime = 2;
            self.gameStateData.room = 1;
            self.gameStateData.round = 1;
            self.gameStateData.lightSource = 0;
            self.gameStateData.charIsDead = false;
            self.gameStateData.speedCounter = 0;

            self.startupGameEngine(self.gameStateData);

        }, "json");
    };

    self.startup = function ()
    {
        //TODO: Check browser compatibility

        $('#txtCommand').keydown(self.keyDownHandler).val("");

        self.sammy.run();

        $('#txtCommand').focus();
    };

    self.startupGameEngine = function()
    {
        // Initialize game engine
        var cnsl = new XConsole("divConsole", "consoleText", 250);
        
        self.gameEngine.initialize(cnsl, self.persistenceManager, self.gameStateData, new XRndNum(), new XCombatEngine());

        self.commandHistory = new XCmdHistory(5);

        self.gameEngine.startup();
    };
    
    self.keyDownHandler = function (e)
    {
        if (e.which == 38) // Up arrow key
        {
            $('#txtCommand').val(self.commandHistory.rotate());

            return;
        }

        if (e.which == 13) // Return key
        {
            var c = $('#txtCommand').val().trim();

            if (c == "")
            {
                c = self.commandHistory.getLastCommand();
            }
            else
            {
                self.commandHistory.add(c);
            }

            $('#txtCommand').val(""); // Clear the input line

            var r = self.gameEngine.processCommand(c);
            
            if (r.exitGame)
            {
                alert("TODO: Game Over");
            }
        }
    };

    self.ajaxErr = function() //TODO: Wire this up to AJAX calls
    {
        alert("An error occurred while communicating with the server. Please retry your last action.");
    };
}