﻿var CLegacyConverter = new function()
{
    var self = this;

    self.getArtifact = function(gameModule, n)
    {
        var x = _.findWhere(gameModule.artifacts, { number: n });

        if (_.isUndefined(x))
        {
            console.log("[ERROR] Invalid Artifact #" + n, "red");
            return null;
        }

        return x;
    };

    self.convert = function(gameModule)
    {
        _.each(gameModule.monsters, function(monster)
        {
            monster.hasBeenSeen = false;
            monster.hardiness = monster.data[0];
            monster.agility = monster.data[1];
            monster.currentGroupSize = monster.data[2];
            monster.originalGroupSize = monster.data[2];
            monster.courage = monster.data[3];
            monster.armorClass = monster.data[6];
            monster.weaponNumber = monster.data[7];
            monster.personality = monster.data[10];
            monster.damageTaken = 0;
            monster.deadBodyNumber = 0; //TODO: Need to determine this value

            if (monster.data[4] == 0)
            {
                monster.locationDetail = {
                    type: CLocationType.hidden,
                    number: 0
                };
            }
            else
            {
                monster.locationDetail = {
                    type: CLocationType.room,
                    number: monster.data[4]
                };
            }
        });

        _.each(gameModule.artifacts, function(artifact)
        {
            artifact.hasBeenSeen = false;
            artifact.isForeignArtifact = false; // Came from the module and not the player
            artifact.weight = artifact.data[2];
            artifact.type = artifact.data[1];
            artifact.openStatus = COpenStatus.cannotBeOpened;

            // Set location detail for the Artifact
            artifact.locationDetail = {};

            if (artifact.data[3] == 0)
            {
                artifact.locationDetail.type = CLocationType.hidden;
                artifact.locationDetail.number = 0;
            }
            else if (artifact.data[3] == -1)
            {
                artifact.locationDetail.type = CLocationType.carriedByPlayer;
                artifact.locationDetail.number = 0;
            }
            else if (artifact.data[3] == -999)
            {
                artifact.locationDetail.type = CLocationType.wornByPlayer;
                artifact.locationDetail.number = 0;
            }
            else if (artifact.data[3] < -1000)
            {
                artifact.locationDetail.type = CLocationType.wornByMonster;
                artifact.locationDetail.number = Math.abs(artifact.data[3] + 1000);
            }
            else if (artifact.data[3] < -1)
            {
                artifact.locationDetail.type = CLocationType.carriedByMonster;
                artifact.locationDetail.number = Math.abs(artifact.data[3] + 1);
            }
            else if (artifact.data[3] > 2000)
            {
                artifact.locationDetail.type = CLocationType.embedded;
                artifact.locationDetail.number = artifact.data[3] - 2000;
            }
            else if (artifact.data[3] > 1000)
            {
                artifact.locationDetail.type = CLocationType.containedInArtifact;
                artifact.locationDetail.number = artifact.data[3] - 1000;
            }
            else
            {
                artifact.locationDetail.type = CLocationType.room;
                artifact.locationDetail.number = artifact.data[3];
            }

            // Set type-specific details
            switch (artifact.data[1])
            {
                case CArtifactType.weapon:
                case CArtifactType.magicWeapon:
                    artifact.dice = artifact.data[6];
                    artifact.sides = artifact.data[7];
                    artifact.weaponType = artifact.data[5];
                    artifact.odds = artifact.data[4];
                    break;

                case CArtifactType.boundMonster:
                    artifact.boundMonsterNumber = artifact.data[4];
                    artifact.keyNumber = artifact.data[5];
                    artifact.guardMonsterNumber = artifact.data[6];

                case CArtifactType.container:
                    artifact.keyNumber = artifact.data[4];
                    artifact.openStatus = COpenStatus.fromData(artifact.data[5], 1, 0);
                    artifact.smashOpenValue = (artifact.data[5] > 1000) ? artifact.data[5] - 1000 : 0;
                    artifact.maxCanHold = artifact.data[7];

                case CArtifactType.disguisedMonster:
                    artifact.disguisedMonsterNumber = artifact.data[4];
                    artifact.effectNumber = artifact.data[5];
                    artifact.numEffects = artifact.data[6];

                case CArtifactType.drinkable:
                case CArtifactType.edible:
                    artifact.openStatus = COpenStatus.fromData(artifact.data[6], 1, 0);
                    artifact.healPoints = artifact.data[4];
                    artifact.amountRemaining = artifact.data[5];
                    artifact.smashOpenValue = (artifact.data[6] > 1000) ? artifact.data[6] - 1000 : 0;
                    break;

                case CArtifactType.lightable:
                    artifact.lightCounter = artifact.data[4];
                    break;

                case CArtifactType.readable:
                    artifact.openStatus = COpenStatus.fromData(artifact.data[6], 1, 0);
                    artifact.effectNumber = artifact.data[4];
                    artifact.numEffects = artifact.data[5];
                    artifact.smashOpenValue = (artifact.data[6] > 1000) ? artifact.data[6] - 1000 : 0;
                    break;

                case CArtifactType.doorGate:
                    artifact.roomBeyond = artifact.data[4];
                    artifact.keyNumber = artifact.data[5];
                    artifact.openStatus = COpenStatus.fromData(artifact.data[6], 0, 1);
                    artifact.isHidden = (artifact.data[7] != 0);
                    artifact.smashOpenValue = (artifact.data[6] > 1000) ? artifact.data[6] - 1000 : 0;

                case CArtifactType.deadBody:
                    artifact.takeableStatus = (artifact.data[4] != 0);

                case CArtifactType.wearable:
                    artifact.armorClass = artifact.data[4];
                    artifact.armorType = artifact.data[5];
            }
        });

        _.each(gameModule.rooms, function(room)
        {
            room.hasBeenSeen = false;
            room.isDark = (room.data[10] == 0);
            room.exits = [];

            for (var r = 0; r < gameModule.numDirections; r++)
            {
                var connection = room.data[r];

                if (connection > 1000)
                {
                    // Door/gate
                    var door = self.getArtifact(gameModule, connection - 1000);

                    if (door != null)
                    {
                        room.exits.push({
                            direction: r,
                            connection: door.roomBeyond,
                            isDoor: true,
                            doorArtifactNumber: door.number
                        });
                    }
                }
                else if (connection > 0)
                {
                    room.exits.push({
                        direction: r,
                        connection: connection,
                        isDoor: false,
                        doorArtifactNumber: null
                    });
                }
            }
        });
    }
}